#include <sps2lib.h>
#include <stdio.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include "PS2Defines.h"
#include "AddRegisters.h"
#include "ctexture.h"
#include "csquaresprite.h"
#include "sps2wrap.h"
#include "pad.h"


//globals
CSquareSprite Sprite( 0, 0, 32, 32);	// A sprite to play with.
CSPS2Manager Manager;					// Manager for Memory stuff etc...
char * pPrimitiveMem;					// pointer to the start of primitive memory.
int PacManMap[16][13];					// This will be our maze.
int xPos=0, yPos=0;	 					// Position of the editable sprite.
int Brightness = 100;					// Brightness of square.
int BrightnessDir = -1;					// Brighness increasing or decreasing.
int SaveSlot = 0;						// The slot to save the maze.

// Function prototypes.
void loadMaze(int SaveSlot);
void saveMaze(int SaveSlot);
void clearMaze();
int fileExists(int SaveSlot);

int main(void)
{

	// Initialize the maze.
	loadMaze(SaveSlot);

	// Init the Control pad, we use PAD_INIT_ANALOGUE so we can use R3 and L3
	if(pad_init (PAD_0, PAD_INIT_ANALOGUE | PAD_INIT_LOCK) == 0)
	{
		printf("Unable to initialise the pads\n");
		pad_cleanup (PAD_0);
		exit(0);
	}

	// Initialise the screen and allocat 640 4K blocks, this is actually 2.5 MB...
	Manager.Initialise(640);

	CTexture Tex(Manager.GetSPS2Handle());	// Texture

	// Load the 256*256 bitmap into SPS2 memory.
	if(!Tex.LoadBitmap("game.bmp", true, false))
	{
		printf("Failed to load texture.");
		return -1;
	}

	// Upload the texture from SPS2 memory into the first free
	// page of GS Memory. 
	Tex.Upload(sps2UScreenGetFirstFreeGSPage(), false);

	do
	{	

		// Update the pad structures
		pad_update (PAD_0);

		// Update Brightness
		Brightness += BrightnessDir;
		if (Brightness>127) {
			BrightnessDir = -BrightnessDir;
		}
		if (Brightness<50) {
			BrightnessDir = -BrightnessDir;
		}

		// Move around the maze...
		if(pad[0].pressed & PAD_UP) {
			if (yPos>0) {
				yPos--;
			}
		}
		if(pad[0].pressed & PAD_DOWN) {
			if (yPos<12) {
				yPos++;
			}
		}
		if(pad[0].pressed & PAD_LEFT) {
			if (xPos>0) {
				xPos--;
			}
		}
		if(pad[0].pressed & PAD_RIGHT) {
			if (xPos<15) {
				xPos++;
			}
		}

		/*
		Edit the maze by masking bits...
		1 = Right
		2 = Up
		4 = Left
		8 = Down
		16 = Small dot
		32 = Large dot

		Basically this means that if bit 0 (1) is set Pacman can go right,
		If bit 1 is set (2) he can go up, bit 2 (4) for left and bit 3 (8) down.
		The small dot (16) is the ordinary pill and the large dot (32) is the big cookie...
		*/

		// Right direction
		if(pad[0].pressed & PAD_CIRCLE) {
			if(PacManMap[xPos][yPos]&0x01) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 1;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 1;
			}
		}

		// Up direction
		if(pad[0].pressed & PAD_TRI) {
			if(PacManMap[xPos][yPos]&0x02) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 2;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 2;
			}
		}

		// Left direction
		if(pad[0].pressed & PAD_SQUARE) {
			if(PacManMap[xPos][yPos]&0x04) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 4;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 4;
			}
		}
		
		// Down direction
		if(pad[0].pressed & PAD_CROSS) {
			if(PacManMap[xPos][yPos]&0x08) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 8;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 8;
			}
		}

		// Small dot
		if(pad[0].pressed & PAD_R1) {
			if(PacManMap[xPos][yPos]&0x10) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 16;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 16;
			}
		}

		// Big dot
		if(pad[0].pressed & PAD_R2) {
			if(PacManMap[xPos][yPos]&0x20) {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] - 32;
			} else {
				PacManMap[xPos][yPos] = PacManMap[xPos][yPos] + 32;
			}
		}



		// Save and load if we switch save slot...
		if(pad[0].pressed & PAD_L1) {
			if (SaveSlot>0) {
				saveMaze(SaveSlot);
				SaveSlot--;
				loadMaze(SaveSlot);
			}
		}

		// Save and load if we switch save slot...
		if(pad[0].pressed & PAD_L2) {
			if (SaveSlot<15) {
				saveMaze(SaveSlot);
				SaveSlot++;
				loadMaze(SaveSlot);
			}
		}


		// Set/reset memory pointer to the start of primitive memory from the Manager.
		// This is where we will add our GIF Tags.
		// Since we are using the Manager, we don't have to worry about the 4K limitation.
		// The limitation is 256 K which is more than enough...
		pPrimitiveMem = (char *)Manager.GetPrim1Base(); 
		
		//clear the screen to the specified color
		Manager.BeginScene(0x00, 0x00, 0x00);

		// Draw the Maze...
		// We are not really using sprites as sprites, more like blocks to draw the maze...
		// That is why we only have one sprite and set coordinates all the time...
		for (int xMap = 0; xMap <16; xMap++) {
			for (int yMap = 0; yMap <13; yMap++) {
				// Set sprite coordinates from top left corner...
				Sprite.setPos(64 + xMap*32, 32 + yMap*32);

				// If bit 3 is set, we move one column to the right in the texture.
				// We mask out all other bits (&0x08) and divide by 8 (>>3), if the bit was set we now have 1, if not 0. 
				// This will be the actual column.
				// Select the row from the lower 3 bits (%8). Range 0 to 7...
				// Sprite.setTexturePos( Column, Row);
				Sprite.setTexturePos(((PacManMap[xMap][yMap]&0x08)>>3), (PacManMap[xMap][yMap]%8));
				Sprite.setColor(0x80, 0x80, 0x80, 0x80);

				// Draw selected block in different color.
				// Actually we are using a different Texture position...
				if (xPos==xMap && yPos==yMap) {
					// Move two columns to the right...
					// Same thing as above but we added 2 to the column...
					Sprite.setTexturePos(((PacManMap[xMap][yMap]&0x08)>>3) + 2, (PacManMap[xMap][yMap]%8));
					Sprite.setColor(Brightness, Brightness, Brightness, 0x80);
				}

				// Add sprite to memory.
				Sprite.addSprite(pPrimitiveMem, 100);

				// Draw small dot ???
				if (PacManMap[xMap][yMap]&0x10) {
					// Set the small dot and draw it.
					Sprite.setTexturePos(4,0);
					// Add sprite to memory.
					Sprite.addSprite(pPrimitiveMem, 200);
				}

				// Draw Large cookie ???
				if (PacManMap[xMap][yMap]&0x20) {
					// Set the large cookie and draw it.
					Sprite.setTexturePos(4,1);
					// Add sprite to memory.
					Sprite.addSprite(pPrimitiveMem, 200);
				}
			}
		}

		// Send GIF Tags to GS to draw sprites.	
		// Let the manager do the job of figuring out the 4K blocks... ;-)
		Manager.FirePacketToGS((char *)Manager.GetPrim1Base(), pPrimitiveMem, (char *)Manager.GetPrim1DmaBase());

		// Print selected location...
		sps2UPrintf(0, "\377800000\376(250,0)xPos: %d", xPos);
		sps2UPrintf(0, "\377008000\376(330,0)yPos: %d", yPos);

		// Print selected Maze slot.
		sps2UPrintf(0, "\377008080\376(0,20)Save");
		sps2UPrintf(0, "\377008080\376(0,35)slot");
		for (int i=0; i<16; i++) {
			sps2UPrintfSetPos(0,70 + i* 20);
			if (i==SaveSlot) {
				sps2UPrintf(0, "\377FFFFFFSlot:%d", i);
			} else {
				sps2UPrintf(0, "\377808080Slot:%d", i);
			}
		}

		// Print instructions...
		sps2UPrintf(0, "\377800080\376(200,15)Exit by pressing Select+Start.");
		sps2UPrintf(0, "\377808000\376(10,448)Move with directional buttons, edit with Triangle, Circle, Cross and Square.");
		sps2UPrintf(0, "\377808000\376(10,462)Use L1 and L2 to switch save slots. R1 for small dot and R2 for large dot.");


		sps2UPrintfRender();

		// Display the buffer we were just drawing to
    	Manager.EndScene();

	// Press start and select together to end the loop	
	}while(!((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)));

	// Write the unsaved maze.
	saveMaze(SaveSlot);

	// Release the pad device and clean up.
	pad_cleanup (PAD_0);
	
	return 0;
}

// Clear maze.
void clearMaze()
{
	// Initialize maze, set everything to zero.
	for (int xMap = 0; xMap <16; xMap++) {
		for (int yMap = 0; yMap <13; yMap++) {
			PacManMap[xMap][yMap] = 0;
		}
	}
}

// Load maze if the file exist, otherwise start with a clear maze.
void loadMaze(int SaveSlot)
{
	if (fileExists(SaveSlot)) {
		char name[256];
		// Load the new maze.
		sprintf (name, "PlacMan%d.maze", SaveSlot);
		FILE * f = fopen(name, "rb");
		fread(PacManMap, sizeof(PacManMap), 1, f);
		fclose(f);
	} else {
		// Clear it.
		clearMaze();
	}
}

// Save maze.
void saveMaze(int SaveSlot)
{
	// Save the current maze
	char name[256];
	sprintf (name, "PlacMan%d.maze", SaveSlot);
	FILE * f = fopen(name, "wb");
	fwrite(PacManMap, sizeof(PacManMap), 1, f);
	fclose(f);
}

// Test to see if a file exist.
int fileExists(int SaveSlot)
{
	char name[256];
	sprintf (name, "PlacMan%d.maze", SaveSlot);
	FILE * f;
	f = fopen(name, "rb");		// Try to open the file
	if (f == NULL) return 0; 	// File doesn't exist
	fclose (f);
	return 1;					// File exists
}	
